#include "Config/CSVConfigParser.h"
#include "Misc/FileHelper.h"
#include "Misc/Paths.h"

CSVConfigParser::CSVConfigParser()
{
    ParsedData.Empty();
}

bool CSVConfigParser::LoadConfig(const FString& FilePath)
{
    ParsedData.Empty();

    // ȡ CSV ļ
    FString FileContent;
    if (!FFileHelper::LoadFileToString(FileContent, *FilePath))
    {
        UE_LOG(LogTemp, Error, TEXT("Failed to load CSV file: %s"), *FilePath);
        return false;
    }

    // н CSV
    TArray<FString> Lines;
    FileContent.ParseIntoArrayLines(Lines);

    for (const FString& Line : Lines)
    {
        TArray<FString> Cells;
        Line.ParseIntoArray(Cells, TEXT(",")); // ԶΪָԪ
        ParsedData.Add(Cells);
    }

    return true;
}

TArray<FString> CSVConfigParser::GetRow(int32 RowIndex) const
{
    if (ParsedData.IsValidIndex(RowIndex))
    {
        return ParsedData[RowIndex];
    }
    return TArray<FString>();
}

TArray<FString> CSVConfigParser::GetColumn(int32 ColumnIndex) const
{
    TArray<FString> Column;
    for (const TArray<FString>& Row : ParsedData)
    {
        if (Row.IsValidIndex(ColumnIndex))
        {
            Column.Add(Row[ColumnIndex]);
        }
    }
    return Column;
}

FString CSVConfigParser::GetCell(int32 RowIndex, int32 ColumnIndex, const FString& DefaultValue) const
{
    if (ParsedData.IsValidIndex(RowIndex) && ParsedData[RowIndex].IsValidIndex(ColumnIndex))
    {
        return ParsedData[RowIndex][ColumnIndex];
    }
    return DefaultValue;
}

FString CSVConfigParser::GetString(const FString& Key, const FString& DefaultValue) const
{
    // Key ʽΪ "RowIndex.ColumnIndex"
    FString RowIndexString, ColumnIndexString;
    if (Key.Split(TEXT("."), &RowIndexString, &ColumnIndexString))
    {
        // תке
        int32 RowIndex = FCString::Atoi(*RowIndexString);
        int32 ColumnIndex = FCString::Atoi(*ColumnIndexString);

        // ʹ GetCell ȡԪֵ
        return GetCell(RowIndex, ColumnIndex, DefaultValue);
    }

    //  Key ʽ󣬷Ĭֵ
    return DefaultValue;
}

    int32 CSVConfigParser::GetInt(const FString & Key, int32 DefaultValue) const
    {
        // ȡֵַתΪ
        FString StringValue = GetString(Key, TEXT(""));
        if (!StringValue.IsEmpty())
        {
            return FCString::Atoi(*StringValue); // תΪ
        }
        return DefaultValue;
    }

    float CSVConfigParser::GetFloat(const FString & Key, float DefaultValue) const
    {
        // ȡֵַתΪ
        FString StringValue = GetString(Key, TEXT(""));
        if (!StringValue.IsEmpty())
        {
            return FCString::Atof(*StringValue); // תΪ
        }
        return DefaultValue;
    }

    bool CSVConfigParser::GetBool(const FString & Key, bool DefaultValue) const
    {
        // ȡֵַתΪֵ
        FString StringValue = GetString(Key, TEXT(""));
        if (!StringValue.IsEmpty())
        {
            return StringValue.Equals(TEXT("true"), ESearchCase::IgnoreCase) || StringValue.Equals(TEXT("1"));
        }
        return DefaultValue;
    }

    TArray<FString> CSVConfigParser::GetArray(const FString & Key) const
    {
        //  Key к
        return GetRow(FCString::Atoi(*Key));
    }

    TSharedPtr<IConfigParser> CSVConfigParser::GetObject(const FString & Key) const
    {
        //  Key к
        TArray<FString> Column = GetColumn(FCString::Atoi(*Key));
        if (Column.Num() > 0)
        {
            TSharedPtr<CSVConfigParser> SubParser = MakeShared<CSVConfigParser>();
            SubParser->ParsedData.Add(Column);
            return SubParser;
        }

        return nullptr;
    }

    void CSVConfigParser::PrintConfig() const
    {
        for (const TArray<FString>& Row : ParsedData)
        {
            FString RowString = FString::Join(Row, TEXT(", "));
            UE_LOG(LogTemp, Log, TEXT("%s"), *RowString);
        }
    }